
# Device group container definition     
DEVICE_GROUP_CND = "com.telekom.iot.orchestrator.deviceGroup"

# Type label name
APPL_TYPE_LABEL_NAME = "type"

# Webhook application type 
APPL_TYPE_WEBHOOK = "webHook"

# Default application type
APPL_TYPE_DEFAULT = APPL_TYPE_WEBHOOK

# Label prefixs for the type specific connection properties
CONN_PROPS_LABEL_PREFIXS = {
    APPL_TYPE_WEBHOOK: "WebHookHttpHeader_"
}

# Description label name
DESCRIPTION_LABEL_NAME = "description"
